@@ -18,6 +18,7 @@ from wsproto.events import (
     Request,
     TextMessage,
 )
+from wsproto.utilities import LocalProtocolError
 
 
 class Websocket:
@@ -62,10 +63,7 @@ class Websocket:
         data = self._connection.send(
             Request(host=addr[0], target=path, extra_headers=headers, subprotocols=subprotocols)
         )
-        try:
-            await self._sock.send_all(data)
-        except AttributeError:
-            await self._sock.send(data)
+        await self._sock.send(data)
 
         event = await self._next_event()
         if not isinstance(event, AcceptConnection):
@@ -102,10 +100,7 @@ class Websocket:
             subprotocol = event.subprotocols[0] if event.subprotocols else None
             msg = AcceptConnection(subprotocol=subprotocol)
         data = self._connection.send(msg)
-        try:
-            await self._sock.send_all(data)
-        except AttributeError:
-            await self._sock.send(data)
+        await self._sock.send(data)
         if not isinstance(msg, AcceptConnection):
             raise ConnectionError("Not accepted", msg)
 
@@ -125,10 +120,7 @@ class Websocket:
 
             if self._sock is None:
                 return CloseConnection(code=500, reason="Socket closed")
-            try:
-                data = await self._sock.receive_some(4096)
-            except AttributeError:
-                data = await self._sock.receive(4096)
+            data = await self._sock.receive(4096)
             if not data:
                 return CloseConnection(code=500, reason="Socket closed")
             self._connection.receive_data(data)
@@ -141,17 +133,17 @@ class Websocket:
         if not sock:
             return
 
-        data = self._connection.send(CloseConnection(code=code, reason=reason))
         try:
-            await sock.send_all(data)
-        except AttributeError:
-            await sock.send(data)
+            data = self._connection.send(CloseConnection(code=code, reason=reason))
+        except LocalProtocolError:
+            # not yet fully open
+            pass
+        else:
+            async with self._send_lock:
+                await sock.send(data)
 
         # No, we don't wait for the correct reply
-        try:
-            await sock.close()
-        except AttributeError:
-            await sock.aclose()
+        await sock.aclose()
 
     async def send(self, data: Union[bytes, str], final: bool = True):
         """
@@ -161,10 +153,7 @@ class Websocket:
         data = MsgType(data=data, message_finished=final)
         async with self._send_lock:
             data = self._connection.send(event=data)
-            try:
-                await self._sock.send_all(data)
-            except AttributeError:
-                await self._sock.send(data)
+            await self._sock.send(data)
 
     def _buffer(self, event: Message):
         """
